;; Time-stamp: <> -*-nasm-*-
;; factorial.asm: This program shows how to call a function recursively. 

; Purpose - Given a number, this program computes the factorial.  For
;           example, the factorial of 3 is 3 * 2 * 1, or 6.  The
;           factorial of 4 is 4 * 3 * 2 * 1, or 24, and so on.

          global    _start 
          global    factorial     ; This is unneeded unless we want to
                                  ;   share this function among other
                                  ;   programs.
          section   .text 
_start:   push      4             ; The factorial takes one argument -
                                  ;   the number we want a factorial
                                  ;   of.  So, it gets pushed.
          call      factorial     ; Run the factorial function.
          add       esp,4         ; Scrubs the parameter that was
                                  ;   pushed on the stack.
          mov       ebx,eax       ; Factorial returns the answer in
                                  ;   EAX, but we want it in EBX to
                                  ;   send it as our exit status.
          mov       eax,1         ; Call the kernel's exit function 
          int       80h 

; This is the actual function definition.
; .type factorial,@function 

factorial:push      ebp           ; Standard function stuff - we have
                                  ;   to restore EBP to its prior
                                  ;   state before returning, so we
                                  ;   have to push it.
          mov       ebp,esp       ; This is because we don't want to
                                  ;   modify the stack pointer, so we
                                  ;   use EBP.
          mov       eax,[ebp+8]   ; This moves the first argument to
                                  ;   EAX, [EBP+4] holds the return
                                  ;   address, and [EBP+8] holds the
                                  ;   first parameter.
          cmp       eax,1         ; If the number is 1, that is our
                                  ;   base case, and we simply return
                                  ;   (1 is already in EAX as the
                                  ;   return value).
          je        end_factorial 
          dec       eax           ; Otherwise, decrease the value.
          push      eax           ; Push it for our call to factorial.
          call      factorial     ; Call factorial.
          mov       ebx,[ebp+8]   ; EAX has the return value, so we
                                  ;   reload our parameter into EBX.
          imul      eax,ebx       ; Multiply that by the result of the
                                  ;   last call to factorial (in EAX) 
                                  ;   the answer is stored in EAX,
                                  ;   which is good since that's where
                                  ;   return values go. 
end_factorial: 
          mov       esp,ebp       ; Standard function return stuff -
          pop       ebp           ;   we have to restore EBP and ESP
                                  ;   to where they were before the
                                  ;   function started.
          ret                     ; Return to the function (this pops
                                  ;   the return value, too).
