;; Time-stamp: <04/03/27 20:18:14 wfz> -*-nasm-*-
;; gnome-example.asm: A GUI program written with the GNOME libraries.
 
; Entry conditions: The user can only click on the "Quit" button or
;                   close the window.
; Exit conditions:  The application will close.
 
; Process: If the user clicks on the "Quit" button, the program will
;          display a dialog asking if they are sure.  If they click
;          Yes, it will close the application.  Otherwise it will
;          continue running.
 
%include  "linux.inc"

          global    main

          section   .text 
;         .type     main,@function 
main:     push      ebp
          mov       ebp,esp

; Initialize GNOME libraries 

          push      dword[ebp+12] ; argv 
          push      dword[ebp+8]  ; argc 
          push      app_version 
          push      app_id 
          lcall     gnome_init 
          add       esp,16        ; recover the stack 

; Create new application window.

          push      app_title     ; Window title 
          push      app_id        ; Application ID 
          lcall     gnome_app_new 
          add       esp,8         ; recover the stack 
          mov       [appPtr],eax  ; save the window pointer 

; Create new button.

          push      button_quit_text; button text 
          lcall     gtk_button_new_with_label 
          add       esp,4         ; recover the stack 
          mov       [btnQuit],eax ; save the button pointer 

; Make the button show up inside the application window.

          push      dword[btnQuit]
          push      dword[appPtr]
          lcall     gnome_app_set_contents 
          add       esp,8 

; Makes the button show up (only after it's window shows up, though).

          push      dword[btnQuit]
          lcall     gtk_widget_show 
          add       esp,4 

; Makes the application window show up.

          push      dword[appPtr]
          call      gtk_widget_show 
          add       esp,4 

; Have GNOME call our delete_handler function whenever a "delete"
; event occurs.

          push      NULL          ; extra data to pass to our 

; function (we don't use any) 

          push      delete_handler; function address to call 
          push      signal_delete_event; name of the signal 
          push      dword[appPtr] ; widget to listen for events on 
          lcall     gtk_signal_connect 
          add       esp,16        ; recover stack 

; Have GNOME call our destroy_handler function whenever a "destroy"
; event occurs.

          push      NULL          ; extra data to pass to our 

; function (we don't use any) 

          push      destroy_handler; function address to call 
          push      signal_destroy; name of the signal 
          push      dword[appPtr] ; widget to listen for events on 
          call      gtk_signal_connect 
          add       esp,16        ; recover stack 

; Have GNOME call our click_handler function whenever a "click" event
; occurs.  Note that the previous signals were listening on the
; application window, while this one is only listening on the button

          push      NULL 
          push      click_handler 
          push      signal_clicked 
          push      dword[btnQuit] 
          call      gtk_signal_connect 
          add       esp,16 

; Transfer control to GNOME.  Everything that happens from here out is
; in reaction to user events, which call signal handlers.  This main
; function just sets up the main window and connects signal handlers,
; and the signal handlers take care of the rest.

          lcall     gtk_main 

; After the program is finished, leave.

          mov       eax,0 
          leave
          ret       
 
; A "destroy" event happens when the widget is being removed.  In this
; case, when the application window is being removed, we simply want
; the event loop to quit.

destroy_handler: 
          push      ebp 
          mov       ebp,esp 

; This causes gtk to exit it's event loop as soon as it can.

          lcall     gtk_main_quit 
          mov       eax,0 
          leave
          ret       

; A "delete" event happens when the application window gets clicked in
; the "x" that you normally use to close a window.

delete_handler: 
          mov       eax,1 
          ret       

; A "click" event happens when the widget gets clicked.

click_handler: 
          push      ebp 
          mov       ebp,esp 

; Create the "Are you sure" dialog 

          push      NULL          ; End of buttons 
          push      GNOME_STOCK_BUTTON_NO; Button 1 
          push      GNOME_STOCK_BUTTON_YES; Button 0 
          push      GNOME_MESSAGE_BOX_QUESTION; Dialog type 
          push      quit_question ; Dialog mesasge 
          lcall     gnome_message_box_new 
          add       esp,16        ; recover stack 

; EAX now holds the pointer to the dialog window.  Setting Modal to 1
; prevents any other user interaction while the dialog is being shown

          push      1 
          push      eax 
          lcall     gtk_window_set_modal 
          pop       eax 
          add       esp,4 

; Now we show the dialog. 

          push      eax 
          call      gtk_widget_show 
          pop       eax 

; This sets up all the necessary signal handlers in order to just show
; the dialog, close it when one of the buttons is clicked, and return
; the number of the button that the user clicked on.  The button
; number is based on the order the buttons were pushed on in the
; gnome_message_box_new function

          push      eax 
          lcall     gnome_dialog_run_and_close 
          add       esp,4 

; Button 0 is the Yes button.  If this is the button they clicked on,
; tell GNOME to quit it's event loop.  Otherwise, do nothing.

          cmp       eax,0 
          jne       click_handler_end 
          call      gtk_main_quit 

click_handler_end: 
          leave
          ret       

          section   .data

;;;GNOME definitions - These were found in the GNOME header files for
;                      the C language and converted into their assembly 
;                      equivalents .

; GNOME Button Names 

GNOME_STOCK_BUTTON_YES: 
          db        "Button_Yes", 0
GNOME_STOCK_BUTTON_NO: 
          db        "Button_No", 0

; Gnome MessageBox Types 

GNOME_MESSAGE_BOX_QUESTION: 
          db        "question", 0

; Standard definition of NULL 

NULL      equ       0 

; GNOME signal definitions 

signal_destroy: 
          db        "destroy", 0
signal_delete_event: 
          db        "delete_event", 0
signal_clicked: 
          db        "clicked", 0

;;;Application-specific definitions 

; Application information 

app_id: 
          db        "gnome-example", 0
app_version: 
          db        "1.000", 0
app_title: 
          db        "Gnome Example Program", 0

; Text for Buttons and windows 

button_quit_text: 
          db        "I Want to Quit the GNOME Example Program", 0
quit_question: 
          db        "Are you sure you want to quit?", 0

          section   .bss 

; Variables to save the created widgets in 

WORD_SIZE equ       4 
appPtr              resb      WORD_SIZE  
btnQuit             resb      WORD_SIZE 
