;; Time-stamp: <> -*-nasm-*-
;; integer-to-string.asm: Convert an integer to a decimal string for display.
 
; Entry conditions: A buffer large enough to hold the largest possible
;                   number
;                   An integer to convert 
 
; Exit conditions:  The buffer will be overwritten with the decimal
;                   string
 
; Variables: 
 
; ECX will hold the count of characters processed 
; EAX will hold the current value 
; EDI will hold the base (10) 
 
ST_VALUE  equ       8 
ST_BUFFER equ       12 

          global    integer2string 

; Normal function beginning 

integer2string: 
          push      ebp 
          mov       ebp,esp 

; Current character count 

          mov       ecx,0 

; Move the value into position 

          mov       eax,[ebp+ST_VALUE] 

; When we divide by 10, the 10 must be in a register or memory
; location

          mov       edi,10 
conversion_loop: 

; Division is actually performed on the combined %edx:%eax register,
; so first clear out %edx

          mov       edx,0 

; Divide %edx:%eax (which are implied) by 10.  Store the quotient in
; %eax and the remainder in %edx (both of which are implied).

          div       edi 

; Quotient is in the right place. %edx has the remainder, which now
; needs to be converted into a number. So, %edx has a number that is 0
; through 9. You could also interpret this as an index on the ASCII
; table starting from the character '0'. The ascii code for '0' plus
; zero is still the ascii code for '0'. The ascii code for '0' plus 1
; is the ascii code for the character '1'. Therefore, the following
; instruction will give us the character for the number stored in %edx

          add       edx,'0' 

; Now we will take this value and push it on the stack. This way, when
; we are done, we can just pop off the characters one-by-one and they
; will be in the right order. Note that we are pushing the whole
; register, but we only need the byte in %dl (the last byte of the
; %edx register) for the character.

          push      edx 

; Increment the digit count 

          inc       ecx 

; Check to see if %eax is zero yet, go to next step if so.

          cmp       eax,0 
          je        end_conversion_loop 

; %eax already has its new value. 

          jmp       conversion_loop 
end_conversion_loop: 

; The string is now on the stack, if we pop it off a character at a
; time we can copy it into the buffer and be done.
; Get the pointer to the buffer in %edx 

          mov       edx,[ebp+ST_BUFFER] 
copy_reversing_loop: 

; We pushed a whole register, but we only need the last byte. So we
; are going to pop off to the entire %eax register, but then only move
; the small part (%al) into the character string.

          pop       eax 
          mov       [edx],al 

; Decreasing %ecx so we know when we are finished 

          dec       ecx 

; Increasing %edx so that it will be pointing to the next byte

          inc       edx 

; Check to see if we are finished 

          cmp       ecx,0 

; If so, jump to the end of the function 

          je        end_copy_reversing_loop 

; Otherwise, repeat the loop 

          jmp       copy_reversing_loop 
end_copy_reversing_loop: 

; Done copying. Now write a null byte and return 

          mov       byte[edx],0 
          mov       esp,ebp 
          pop       ebp 
          ret       
