;; Time-stamp: <> -*-nasm-*-
;; power.asm:  Program to illustrate how functions work 

;           This program will compute the value of 
;           2^3 + 5^2 
 
; Everything in the main program is stored in registers, so the data
; section doesn't have anything.

          global    _start 

          section   .text 
_start:   push      3             ; Push second argument.
          push      2             ; Push first argument.
          call      power         ; Call the function.
          add       esp,8         ; Move the stack pointer back.
          push      eax           ; Save the first answer before 
                                  ;   calling the next function.
          push      2             ; Push second argument.
          push      5             ; Push first argument.
          call      power         ; Call the function.
          add       esp,8         ; Move the stack pointer back.
          pop       ebx           ; The second answer is already 
                                  ;   in EAX.  We saved the first 
                                  ;   answer onto the stack, so now we
                                  ;   can just pop it out into EBX.
          add       ebx,eax       ; Add them together  the result is
                                  ;   in EBX.
          mov       eax,1         ; Exit (EBX is returned).
          int       80h 

; Purpose:          This function is used to compute the value of a
;                   number raised to a power.
; Entry conditions: First argument - the base number.
;                   Second argument - the power to raise it to.
; Exit conditions:  Will give the result as a return value. 
; Notes:            The power must be 1 or greater.

; Variables: 
;                   EBX - holds the base number 
;                   ECX - holds the power  
;                   [EBP-4] - holds the current result 
;                   EAX is used for temporary storage 
 
; .type power, @function 

power:    push      ebp           ; Save old base pointer.
          mov       ebp,esp       ; Make stack pointer the base
                                  ;   pointer.
          sub       esp,4         ; Get room for our local storage.
          mov       ebx,[ebp+8]   ; Put first argument in EAX.
          mov       ecx,[ebp+12]  ; Put second argument in ECX.
          mov       [ebp-4],ebx   ; Store current result.
power_loop_start: 
          cmp       ecx,1         ; If the power is 1, we are done.
          je        end_power 
          mov       eax,[ebp-4]   ; Move the current result into EAX. 
          imul      eax,ebx       ; Multiply the current result by the
                                  ;   base number.
          mov       [ebp-4],eax   ; Store the current result.
          dec       ecx           ; Decrease the power.
          jmp       power_loop_start; Run for the next power.
end_power: 
          mov       eax,[ebp-4]   ; Return value goes in EAX. 
          mov       esp,ebp       ; Restore the stack pointer.
          pop       ebp           ; Restore the base pointer.
          ret       
